@@ -5,7 +5,7 @@
 // gcc -I . -W -Wall -g -O0 -o abuserdap abuserdap.c report_error.c -lcurl (beta)
 
 /*
-Copyright (C) 2019 Alessandro Vesely
+Copyright (C) 2019-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -24,22 +24,27 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
-static const char usage[] = "[opt] rdap-url-or-file\n"
+static const char usage[] = "[opt] rdap-url-or-file-or-IP\n"
 	"where\n"
 	"	opt is one or more of:\n"
-	"	    h  display help and exit\n"
-	"	    l  use syslog (LOG_MAIL), not stderr\n"
-	"       s  display the source on the second line\n"
-	"	    v  for increased verbosity\n"
-	"       x  exclude addresses in file\n"
+	"	   -h      display help and exit\n"
+	"	   -l      use syslog (LOG_MAIL), not stderr\n"
+	"	   -m int  for increased max redirections (from 3)\n"
+	"	   -x file exclude addresses in file\n"
+	"	   -@      display collected exclusion tokens\n"
+	"	   -s      display the data source\n"
+	"	   -X      display the count of exclusions\n"
+	"	   -v      for increased verbosity\n"
 	"	  or\n"
 	"	    --help (same as -h) or --version\n"
-	"	rdap-url-or-file is either the complete URL as per rfc7482\n"
-	"	or a filename, if -f was used, containing json data\n\n"
-	"and rdap-uri can be based on:";
+	"	rdap-url-or-file-or-IP is either the URL as per rfc7482\n"
+	"	or a file containing json data or an IP address.\n\n"
+	"An rdap-uri can be based on:";
+
+static const char bootstrap_uri[] = "http://rdap.arin.net/bootstrap/ip/";
 
 static const char *rdap_uri[] = {
-	"http://rdap.arin.net/bootstrap/ip/",
+	bootstrap_uri,
 	"http://rdap.arin.net/registry/ip/",
 	"https://rdap.db.ripe.net/ip/",
 	"https://rdap.apnic.net/ip/",
@@ -71,105 +76,72 @@ static const char *rdap_uri[] = {
 #define JSMN_STRICT // real JSON
 #include "jsmn.h"
 
+#include <assert.h>
+
 typedef struct global
 {
 	char *arg;
+	char *xclude;
 	char *location;
-	FILE *xfp;        // exclusion file (see grep_v())
+	FILE *xfp;           // exclusion file (see grep_v())
 	size_t datalen;
 	app_private ap;
-	int source_print; // opt -s
-	int verbose;
+	int max_redir;       // default 3
+	int excluded;        // number of excluded addresses
+	int searched;        // calls of grep_v
 	int rtc;
+	int token_print: 1;  // opt -@
+	int source_print: 1; // opt -s
+	int ex_print: 1;     // opt -X
+	int nu: 5;
+	int verbose: 8;
 } global;
 
-static char *read_file(global *glo)
+static char *read_file(global *glo, size_t st_size)
 {
 	char *fname = glo->arg;
-	struct stat st;
-	int rc;
-	if ((rc = stat(fname, &st)) == 0 && (st.st_mode & S_IFMT) == S_IFREG)
+	char *data = malloc(st_size + 1);
+	if (data)
 	{
-		char *data = malloc(st.st_size + 1);
-		if (data)
+		FILE *fp = fopen(fname, "r");
+		int good = 1;
+		if (fp)
 		{
-			FILE *fp = fopen(fname, "r");
-			int good = 1;
-			if (fp)
+			if (fread(data, st_size, 1, fp) == 1)
 			{
-				if (fread(data, st.st_size, 1, fp) == 1)
-				{
-					data[st.st_size] = 0;
-					if (glo->verbose >= 1)
-						report_error(&glo->ap, LOG_INFO,
-							"Read %s\n", fname);
-				}
-				else
-				{
-					report_error(&glo->ap, LOG_ERR,
-						"cannot read %s: %s\n", fname, strerror(errno));
-					good = 0;
-				}
-				fclose(fp);
+				data[st_size] = 0;
+				if (glo->verbose >= 1)
+					report_error(&glo->ap, LOG_INFO,
+						"Read %s\n", fname);
 			}
 			else
 			{
 				report_error(&glo->ap, LOG_ERR,
-					"cannot open %s: %s\n", fname, strerror(errno));
+					"cannot read %s: %s\n", fname, strerror(errno));
 				good = 0;
 			}
-
-			if (good)
-				glo->datalen = st.st_size;
-			else
-			{
-				free(data);
-				data = NULL;
-				glo->rtc = 2;
-			}
-
-			return data;
+			fclose(fp);
 		}
-		report_error(&glo->ap, LOG_CRIT, "Memory fault!!\n");
-		abort();
-	}
-
-	report_error(&glo->ap, LOG_ERR, "%s not %s: %s\n",
-		fname, rc? "found": "a regular file", strerror(errno));
-	glo->rtc = 2;
-	return NULL;
-}
-
-static int grep_v(global *glo, char *addr, size_t addrlen)
-{
-	FILE *fp = glo->xfp;
-	int rtc = 0;
-
-	rewind(fp);
-	char buf[4096], *addrz = strndup(addr, addrlen), *s;
-	if (addrz == NULL)
-		abort();
-
-	while ((s = fgets(buf, sizeof buf, fp)) != NULL)
-	{
-		int ch;
-		while ((ch = *(unsigned char*)s) != 0 && isspace(ch))
-			++s;
-		if (ch == '#')
-			continue;
-
-		if (strstr(s, addrz))
+		else
 		{
-			if (glo->verbose >= 2)
-				report_error(&glo->ap, LOG_INFO,
-					"email \"%s\" excluded by option\n", addrz);
-			rtc = 1;
-			break;
+			report_error(&glo->ap, LOG_ERR,
+				"cannot open %s: %s\n", fname, strerror(errno));
+			good = 0;
 		}
-	}
 
-	free(addrz);
-	return rtc;
+		if (good)
+			glo->datalen = st_size;
+		else
+		{
+			free(data);
+			data = NULL;
+			glo->rtc = 2;
+		}
+
+		return data;
+	}
+	report_error(&glo->ap, LOG_CRIT, "Memory fault!!\n");
+	abort();
 }
 
 typedef struct query_data
@@ -355,7 +327,7 @@ static char *rdap_query(global *glo)
 		goto cleanup;
 	}
 
-	for (int redirect = 0; redirect < 2; ++redirect) // maybe twice
+	for (int redirect = 0; ; ++redirect)
 	{
 		res = curl_easy_setopt(curl, CURLOPT_URL, url);
 		if (res == CURLE_OK)
@@ -396,7 +368,7 @@ static char *rdap_query(global *glo)
 						q.location? q.location: "NO LOCATION", redirect + 1);
 
 				q.b_used = 0;
-				if (redirect > 0 || !q.location)
+				if (redirect > glo->max_redir || !q.location)
 					break;
 
 				url = q.location;
@@ -455,6 +427,33 @@ cleanup:
 	return q.buffer;
 }
 
+static char *file_or_IP(global *glo)
+{
+	char *forip = glo->arg;
+	struct stat st;
+	int rc;
+	if ((rc = stat(forip, &st)) == 0 && (st.st_mode & S_IFMT) == S_IFREG)
+	{
+		return read_file(glo, st.st_size);
+	}
+
+	size_t foriplen = strlen(forip);
+	if (strspn(forip, "0123456789.:abcdefABCDEF") == foriplen)
+	{
+		char arg[foriplen + sizeof bootstrap_uri];
+		strcpy(&arg[0], bootstrap_uri);
+		strcat(&arg[sizeof bootstrap_uri - 1], forip);
+		glo->arg = &arg[0];
+		return rdap_query(glo);
+	}
+
+	report_error(&glo->ap, LOG_ERR, "%s not an IP nor %s file: %s\n",
+		forip, rc? "existing": "a regular", strerror(errno));
+	glo->rtc = 2;
+	return NULL;
+}
+
+
 typedef struct addrlst
 {
 	struct addrlst *next;
@@ -466,10 +465,112 @@ typedef struct json_data
 	global *glo;
 	char *data;
 	jsmntok_t *tok;
-	addrlst *albase;
-	unsigned int addrcount;
+	addrlst *albase, *tokbase;
+	unsigned int addrcount, tokcount;
 } json_data;
 
+static void insert_addrlist(app_private *ap,
+	addrlst **old, char *addr, size_t addrlen)
+/*
+* insert a new element at old, abort if out of memory
+*/
+{
+	assert(old);
+
+	addrlst *new_addr = malloc(sizeof(addrlst) + addrlen + 1);
+	if (new_addr == NULL)
+	{
+		free(*old);
+		report_error(ap, LOG_CRIT, "Memory fault\n");
+		abort();
+	}
+	new_addr->next = *old;
+	memcpy(&new_addr->addr[0], addr, addrlen);
+	new_addr->addr[addrlen] = 0;
+	*old = new_addr;
+}
+
+static int search_addrlist(addrlst **old, char *addr, size_t addrlen)
+{
+	while (*old != NULL)
+	{
+		if (addrlen == strlen((*old)->addr) &&
+			strncasecmp(addr, (*old)->addr, addrlen) == 0)
+				return 1; // found
+		old = &(*old)->next;
+	}
+
+	return 0;
+}
+
+static int grep_v(json_data *jd, char *addr, size_t addrlen)
+/*
+* Check exclusion file, return 0 if not excluded.
+*/
+{
+	global *glo = jd->glo;
+	FILE *fp = glo->xfp;
+	int rtc = 0;
+
+	if (glo->searched > 0 && fseek(fp, 0L, SEEK_SET))
+	{
+		report_error(&glo->ap, LOG_CRIT,
+			"cannot rewind %s: %s\n",
+			glo->xclude, strerror(errno));
+		return 0;
+	}
+
+	++glo->searched;
+	char buf[4096], *addrz = strndup(addr, addrlen), *s;
+	if (addrz == NULL)
+		abort();
+
+	while ((s = fgets(buf, sizeof buf, fp)) != NULL)
+	{
+		int ch;
+		while ((ch = *(unsigned char*)s) != 0 && isspace(ch))
+			++s;
+		if (ch == 0 || ch == '#') // comment
+			continue;
+
+		char *tok = NULL;
+		size_t toklen = 0;
+		if (ch == '@') // token
+		{
+			tok = ++s;
+			while ((ch = *(unsigned char*)s) != 0 && !isspace(ch))
+				++s;
+			if (ch == 0)
+				continue;
+
+			toklen = s - tok;
+			*s++ = 0;
+			while ((ch = *(unsigned char*)s) != 0 && isspace(ch))
+				++s;
+		}
+
+		if (strstr(s, addrz))
+		{
+			glo->excluded += 1;
+			if (glo->verbose >= 2)
+				report_error(&glo->ap, LOG_INFO,
+					"email \"%s\" excluded by option, token=%s\n",
+					addrz, tok? tok: "NULL");
+
+			if (tok && glo->token_print &&
+				search_addrlist(&jd->tokbase, tok, toklen) == 0)
+					insert_addrlist(&jd->glo->ap,
+						&jd->tokbase, tok, toklen);
+
+			rtc = 1;
+			break;
+		}
+	}
+
+	free(addrz);
+	return rtc;
+}
+
 static void add_addrlst(json_data *jd, jsmntok_t *tok)
 {
 	if (tok == NULL)
@@ -483,61 +584,52 @@ static void add_addrlst(json_data *jd, jsmntok_t *tok)
 	/*
 	* discard duplicate address (maybe one has {"pref": 1})
 	*/
-	addrlst **old = &jd->albase;
-	while (*old != NULL)
+	if (search_addrlist(&jd->albase, addr, addrlen))
 	{
-		if (addrlen == strlen((*old)->addr) &&
-			strncasecmp(addr, (*old)->addr, addrlen) == 0)
-		{
-			if (jd->glo->verbose >= 2)
-				report_error(&jd->glo->ap, LOG_INFO,
-					"email \"%.*s\" found again\n", (int)addrlen, addr);
-			return;
-		}
-		old = &(*old)->next;
+		if (jd->glo->verbose >= 2)
+			report_error(&jd->glo->ap, LOG_INFO,
+				"email \"%.*s\" found again\n", (int)addrlen, addr);
+		return;
 	}
 
 	/*
 	* Check exclusion list
 	* TODO: move this check at the end.
 	*/
-	if (jd->glo->xfp && grep_v(jd->glo, addr, addrlen))
+	if (jd->glo->xfp && grep_v(jd, addr, addrlen))
 		return;
 
 	/*
-	* add address to the last NULL pointer
+	* insert new address
 	*/
-	addrlst *new_addr = malloc(sizeof(addrlst) + addrlen + 1);
-	if (new_addr == NULL)
-	{
-		free(jd->albase);
-		report_error(&jd->glo->ap, LOG_CRIT,
-			"Memory fault at address %u\n", jd->addrcount);
-		abort();
-	}
-	new_addr->next = NULL;
-	memcpy(&new_addr->addr[0], addr, addrlen);
-	new_addr->addr[addrlen] = 0;
-	*old = new_addr;
+	insert_addrlist(&jd->glo->ap, &jd->albase, addr, addrlen);
 	jd->addrcount += 1;
 	if (jd->glo->verbose >= 2)
 		report_error(&jd->glo->ap, LOG_INFO, "address %u: %s\n",
-			jd->addrcount, new_addr->addr);
+			jd->addrcount, jd->albase->addr);
 };
 
-static int print_addrlst(json_data *jd)
+static int print_addrlst(addrlst **base, char const *sep)
+/*
+* Print one line with all entries (possibly 0) and free the list
+*/
 {
+	assert(base);
+
 	int rtc = 0;
-	addrlst *prt = jd->albase;
+	addrlst *prt = *base;
 	while (prt)
 	{
 		addrlst *next = prt->next;
-		printf("%s%s", prt->addr, next? ", ": "\n");
+		printf("%s%s", prt->addr, next? sep: "\n");
 		free(prt);
 		prt = next;
 		++rtc;
 	}
-	jd->albase = NULL;
+	*base = NULL;
+
+	if (rtc == 0)
+		putchar('\n');
 
 	return rtc;
 }
@@ -627,9 +719,11 @@ static char const *tokstring(char *data, jsmntok_t *tok)
 	return tok && data? data + tok->start: "";
 }
 
-static int print_abuse_email(global *glo, char *data)
+static void print_abuse_email(global *glo, char *data)
+/*
+* Parse data and print 
+*/
 {
-	int rtc = 0;
 	jsmn_parser parser;
 	jsmn_init(&parser);
 
@@ -783,10 +877,17 @@ static int print_abuse_email(global *glo, char *data)
 				}
 			}
 
-			rtc = print_addrlst(&jd);
+			int rtc = print_addrlst(&jd.albase, ", ");
 			if (glo->verbose)
 				report_error(&glo->ap, LOG_INFO,
 					"%d abuse email address(es) found\n", rtc);
+			if (glo->token_print)
+			{
+				rtc = print_addrlst(&jd.tokbase, " ");
+				if (glo->verbose)
+					report_error(&glo->ap, LOG_INFO,
+						"%d exclude tokens collected\n", rtc);
+			}
 		}
 		else
 			report_error(&glo->ap, LOG_CRIT,
@@ -800,8 +901,6 @@ static int print_abuse_email(global *glo, char *data)
 			ntok == JSMN_ERROR_NOMEM? "NOMEM":
 			ntok == JSMN_ERROR_INVAL? "INVALID":
 			ntok == JSMN_ERROR_PART? "PART": "?", glo->arg);
-
-	return rtc;
 }
 
 static int check_arg(char **dest, char *src, int opt)
@@ -820,7 +919,6 @@ static int check_arg(char **dest, char *src, int opt)
 
 int main(int argc, char *argv[])
 {
-	char *xclude = NULL;
 	global glo;
 
 	memset(&glo, 0, sizeof glo);
@@ -829,6 +927,9 @@ int main(int argc, char *argv[])
 	else
 		glo.ap.err_prefix = argv[0];
 
+	// bootstrap may lead to RIR, then RIR may delegate to national
+	glo.max_redir = 3;
+
 	for (int i = 1; i < argc;)
 	{
 		char *a = argv[i];
@@ -858,12 +959,21 @@ int main(int argc, char *argv[])
 				for (unsigned j = 1; (ch = a[j]) != 0; ++j)
 					switch (ch)
 					{
+						case '@':
+							glo.token_print = 1;
+							break;
+						case 'X':
+							glo.ex_print = 1;
+							break;
 						case 'l':
 						{
 							openlog(glo.ap.err_prefix, LOG_PID,
 								glo.ap.mode = LOG_DAEMON);
 							break;
 						}
+						case 'm':
+							++glo.max_redir;
+							break;
 						case 's':
 							glo.source_print = 1;
 							break;
@@ -871,7 +981,7 @@ int main(int argc, char *argv[])
 							++glo.verbose;
 							break;
 						case 'x':
-							if (check_arg(&xclude, argv[++next_arg], ch))
+							if (check_arg(&glo.xclude, argv[++next_arg], ch))
 								return -1;
 							break;
 						default:
@@ -897,13 +1007,13 @@ int main(int argc, char *argv[])
 
 	if (glo.arg)
 	{
-		if (xclude)
+		if (glo.xclude)
 		{
-			glo.xfp = fopen(xclude, "r");
+			glo.xfp = fopen(glo.xclude, "r");
 			if (glo.xfp == NULL)
 			{
 				report_error(&glo.ap, LOG_ERR,
-					"cannot open %s: %s\n", xclude, strerror(errno));
+					"cannot open %s: %s\n", glo.xclude, strerror(errno));
 				return 2;
 			}
 		}
@@ -918,18 +1028,16 @@ int main(int argc, char *argv[])
 		}
 		else
 		{
-			data = read_file(&glo);
+			data = file_or_IP(&glo);
 		}
 			
 		if (data)
 		{
-			int rtc = print_abuse_email(&glo, data);
+			print_abuse_email(&glo, data);
 			if (glo.source_print)
-			{
-				if (rtc == 0)
-					putchar('\n');
 				puts(glo.arg);
-			}
+			if (glo.ex_print)
+				printf("%d\n", glo.excluded);
 			free(data);
 			free(glo.location);
 		}
