@@ -2,7 +2,7 @@
 * dbstruct.h
 
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -62,13 +62,14 @@ typedef struct ip_data_t
 */
 typedef struct bynumber_descr_t
 {
-	uint64_t caught_cnt, rec_cnt, // set on block record creation/update
+	uint64_t caught_cnt, rec_cnt,  // set on block record creation/update
 		block_cnt, decay_add_cnt;  // set on block record deletion
-	time_t created,               // this record creation
+	time_t created,                // this record creation
 		last_added,                // set on block record creation/update
 		last_removed;              // set on block record deletion
-	unsigned char nu[7];          // reserved --must be 0
-	unsigned char chk;            // CHK_SIGNATURE, for loose type checking
+	unsigned char report_category; // external category added in v. 1.07
+	unsigned char nu[6];           // reserved --must be 0
+	unsigned char chk;             // CHK_SIGNATURE, for loose type checking
 	char descr[/*IPQBDB_DESCR_MAX_LENGTH*/];
 } bynumber_descr_t;
 
@@ -99,7 +100,7 @@ int close_dbenv(DB_ENV *db_env, int cleanup);
 
 
 enum
-{ get_descr_quoted = 1, get_descr_add_id = 2 };
+{ get_descr_quoted = 1, get_descr_add_id = 2, get_descr_add_category = 16 };
 int get_descr(DB *db, int id, int opt, char *dest, size_t size);
 int last_descr_ndx(DB *db1);
 int get_reason_id(int *reason_id, char *reason_string,
@@ -112,14 +113,15 @@ int open_descrdb(char *fname, DB_ENV *db_env, DB** db1, DB** db2);
 int check_whitelist(DB *, uint32_t ip, double *decay);
 
 typedef void (*write_block_cb)(uint32_t ip_addr, int reason_id,
+	int write_block_force,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d);
 enum
 { write_block_force_decay = 1, write_block_force_probability = 2,
-	write_block_force_reason = 4};
+	write_block_force_reason = 4,
+	write_block_force_add_category = 16 };
 int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 	int* /* I/O */ probability, int force, write_block_cb verbose_fn, DB *db_d);
 
 void fputs_initial_count_help(void);
 
-int caught_signal; // defined in the main module, tested while opening db
 #endif // DBSTRUCT_H_INCLUDED
