@@ -2,7 +2,7 @@
 * Ipq Berkeley db Daemon  BAN - ibd-ban
 * written by ale in milano on 13 sep 2008
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -58,64 +58,13 @@ static app_private ap;
 static char const err_prefix[] = "ibd-ban";
 #include "logline_written_block.h"
 
-int caught_signal;
-
 #if !defined NDEBUG
 #define TRACE(x...) while (ap.mode == 0) { fprintf(stderr, ## x); break; }
 #else
 #define TRACE(x...)
 #endif
 
-static void sig_catcher(int sig)
-{
-#if !defined(NDEBUG)
-	if (ap.mode == 0)
-	{
-		char buf[80];
-		unsigned s = snprintf(buf, sizeof buf,
-			"%s [%d]: received signal %s\n",
-			err_prefix, (int)getpid(), strsignal(sig));
-		if (s >= sizeof buf)
-		{
-			s = sizeof buf;
-			buf[s - 1] = '\n';
-		}
-		write(2, buf, s);
-	}
-#endif
-	if (sig == SIGABRT)
-		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
-	caught_signal = sig;
-}
-
-static int setsigs(void)
-{
-	int rtc;
-	struct sigaction act;
-	memset(&act, 0, sizeof act);
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-	act.sa_handler = sig_catcher;
-
-	rtc =
-		sigaction(SIGPIPE, &act, NULL) ||
-		sigaction(SIGILL, &act, NULL)  ||
-		sigaction(SIGHUP, &act, NULL)  ||
-		sigaction(SIGINT, &act, NULL)  ||
-		sigaction(SIGQUIT, &act, NULL) ||
-		sigaction(SIGPWR,  &act, NULL) ||
-		sigaction(SIGABRT, &act, NULL) ||
-		sigaction(SIGTERM, &act, NULL);
-
-	act.sa_handler = SIG_IGN;
-	rtc |=
-		sigaction(SIGALRM, &act, NULL) ||
-		sigaction(SIGIO,   &act, NULL) ||
-		sigaction(SIGUSR1, &act, NULL) ||
-		sigaction(SIGUSR2, &act, NULL);
-//		sigaction(SIGEMT,  &act, NULL);
-	return rtc;
-}
+#include "setsig_func.h"
 
 static int initial_count = IPQBDB_INITIAL_COUNT;
 static double initial_decay = IPQBDB_INITIAL_DECAY;
@@ -126,6 +75,7 @@ static char *reason_string = "0";
 static char **ip_address;
 static int version_opt, help_opt, syslog_opt, cleanup_opt, verbose = -1;
 static int force_probability_opt, force_decay_opt, force_reason_opt;
+static int force_add_category_opt; // not a "force" option, but passed like one
 static int exec_connkill_opt;
 static struct poptOption opttab[] =
 {
@@ -143,6 +93,8 @@ static struct poptOption opttab[] =
 	"Set initial probability count", "integer"},
 	{"force-probability", '\0', POPT_ARG_NONE, &force_probability_opt, 0,
 	"Change the probability even if the current value is higher", NULL},
+	{"log-category", '\0', POPT_ARG_NONE, &force_add_category_opt, 0,
+	"Enable logging the report category", NULL},
 	{"exec-connkill", 'e', POPT_ARG_NONE, &exec_connkill_opt, 0,
 	"Execute connection kill if probability reaches 100%", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
@@ -250,7 +202,7 @@ static void tab_printtime(char const *hdr, time_t ref, time_t now)
 	struct tm tm = *localtime(&ref);
 	unsigned elapsed = now - ref;
 	unsigned dd, hh, mm;
-	char buf[64], *p = &buf[0], buf2[16];;
+	char buf[64], *p = &buf[0], buf2[32];;
 
 	memset(buf, ' ', sizeof buf);
 	dd = elapsed / (24UL * 3600UL);
@@ -276,11 +228,13 @@ static void tab_printtime(char const *hdr, time_t ref, time_t now)
 }
 
 static void display_written_block(uint32_t ip_addr, int reason_id,
+	int write_block_force,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d)
 {
 	// if syslog_opt, log a line anyway (unlikely given -v).
 	if (ap.mode)
-		logline_written_block(ip_addr, reason_id, ip_data, old_data, db_d);
+		logline_written_block(ip_addr, reason_id,
+			write_block_force, ip_data, old_data, db_d);
 
 	struct in_addr in;
 	char descr[19 /* IPQBDB_DESCR_MAX_LENGTH */];
@@ -350,6 +304,7 @@ write_block_display(DB *db, uint32_t ip_addr, double decay, int reason_id, DB *d
 	if (force_probability_opt) force |= write_block_force_probability;
 	if (force_decay_opt) force |= write_block_force_decay;
 	if (force_reason_opt) force |= write_block_force_reason;
+	if (force_add_category_opt) force |= write_block_force_add_category;
 
 	if (write_block(db, ip_addr, decay, reason_id, &probability, force,
 		verbose? display_written_block:
@@ -388,8 +343,8 @@ int main(int argc, char const *argv[])
 	rtc = poptGetNextOpt(opt);
 	if (rtc != -1)
 	{
-		fprintf(stderr, "%s: %s\n",
-			err_prefix, poptStrerror(rtc));
+		fprintf(stderr, "%s: %s at %s\n",
+			err_prefix, poptStrerror(rtc), poptBadOption(opt, 0));
 		errs = 1;
 	}
 	else
@@ -468,12 +423,10 @@ int main(int argc, char const *argv[])
 				rtc = 1;
 		}
 
-		if (reason_id == -1 || verbose)
-		{
-			descr_fname = database_fname(db_descr_name, &ap);
-			if (descr_fname == NULL)
-				rtc = 1;
-		}
+		// need descrdb anyway, for write_block()
+		descr_fname = database_fname(db_descr_name, &ap);
+		if (descr_fname == NULL)
+			rtc = 1;
 
 		if (rtc)
 		{
