@@ -3,7 +3,7 @@
 * written by ale in milano on 15 nov 2008
 * delete and/or list records in a block.db
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -362,59 +362,7 @@ static const char *increase_secs(histog *h, int target, time_t secs)
 
 static app_private ap;
 static char const err_prefix[] = "ibd-del";
-int caught_signal;
-
-
-static void sig_catcher(int sig)
-{
-#if !defined(NDEBUG)
-	if (ap.mode == 0)
-	{
-		char buf[80];
-		unsigned s = snprintf(buf, sizeof buf,
-			"%s [%d]: received signal %s\n",
-			err_prefix, (int)getpid(), strsignal(sig));
-		if (s >= sizeof buf)
-		{
-			s = sizeof buf;
-			buf[s - 1] = '\n';
-		}
-		write(2, buf, s);
-	}
-#endif
-	if (sig == SIGABRT)
-		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
-	caught_signal = sig;
-}
-
-static int setsigs(void)
-{
-	int rtc;
-	struct sigaction act;
-	memset(&act, 0, sizeof act);
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-	act.sa_handler = sig_catcher;
-
-	rtc =
-		sigaction(SIGPIPE, &act, NULL) ||
-		sigaction(SIGILL, &act, NULL)  ||
-		sigaction(SIGHUP, &act, NULL)  ||
-		sigaction(SIGINT, &act, NULL)  ||
-		sigaction(SIGQUIT, &act, NULL) ||
-		sigaction(SIGPWR,  &act, NULL) ||
-		sigaction(SIGABRT, &act, NULL) ||
-		sigaction(SIGTERM, &act, NULL);
-
-	act.sa_handler = SIG_IGN;
-	rtc |=
-		sigaction(SIGALRM, &act, NULL) ||
-		sigaction(SIGIO,   &act, NULL) ||
-		sigaction(SIGUSR1, &act, NULL) ||
-		sigaction(SIGUSR2, &act, NULL);
-//		sigaction(SIGEMT,  &act, NULL);
-	return rtc;
-}
+#include "setsig_func.h"
 
 static char *db_block_name = IPQBDB_DATABASE_NAME;
 static char *db_descr_name = IPQBDB_DESCR_DATABASE_NAME;
@@ -634,6 +582,9 @@ static int get_seconds(time_t *secs, char *opt_time)
 	{
 		switch (toupper(*(unsigned char*)t))
 		{
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wimplicit-fallthrough="
+
 			case 'D':
 				l *= 24ULL;
 				// all through...
@@ -645,6 +596,8 @@ static int get_seconds(time_t *secs, char *opt_time)
 				++t;
 			default:
 				break;
+
+#pragma GCC diagnostic pop
 		}
 	}
 
@@ -667,6 +620,9 @@ static int get_max_records(void)
 	{
 		switch (toupper(*(unsigned char*)t))
 		{
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wimplicit-fallthrough="
+
 			case 'G':
 				l *= 1024ULL;
 				// all through...
@@ -677,6 +633,8 @@ static int get_max_records(void)
 				++t;
 			default:
 				break;
+
+#pragma GCC diagnostic push
 		}
 	}
 
@@ -1053,18 +1011,24 @@ static int do_stat(DB *db)
 		{
 			char buf1[TIME_STRING_BUFSIZE],
 				buf2[TIME_STRING_BUFSIZE],
-				buf3[TIME_STRING_BUFSIZE];
+				buf3[TIME_STRING_BUFSIZE],
+				catbuf[4];
 
 			if (verbose && !total_selected)
-				printf("%11s %11s %11s %8s %8s %11s %8s %s\n",
+				printf("%11s %11s %11s %8s %8s %11s %8s%3s %s\n",
 					"CAUGHT", "BLOCKED", "ADD REC", "LAST ADD", "LAST DEL",
-					"THRESHOLD", "CREATED", "DESCRIPTION");
+					"THRESHOLD", "CREATED", "C", "DESCRIPTION [n]");
+			if (u.drec.report_category)
+				sprintf(catbuf, "%3d", u.drec.report_category);
+			else
+				catbuf[0] = 0;
 			printf("%'11" PRIi64 " %'11" PRIi64 " %'11" PRIi64
-				" %8s %8s" " %'11" PRIi64 " %8s %s [%d]\n",
+				" %8s %8s" " %'11" PRIi64 " %8s%3s %s [%d]\n",
 				u.drec.caught_cnt, u.drec.block_cnt, u.drec.rec_cnt,
 				time_string(u.drec.last_added, now, buf1),
 				time_string(u.drec.last_removed, now, buf2), u.drec.decay_add_cnt,
-				time_string(u.drec.created, now, buf3), u.drec.descr, descr_id);
+				time_string(u.drec.created, now, buf3),
+				catbuf, u.drec.descr, descr_id);
 			total_selected += 1;
 			caught_cnt += u.drec.caught_cnt;
 			block_cnt += u.drec.block_cnt;
