@@ -2,7 +2,7 @@
 * ipq berkeley db daemon emitting verdicts - ibd-judge
 * written by ale in milano on 10 sep 2008
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -94,7 +94,7 @@ does not receive a stop of some sort.
 
 static app_private ap;
 static char const err_prefix[] = "ibd-judge";
-//static int caught_signal;
+#include "setsig_func.h"
 
 // visible for TESTjudge
 int verbose = -1;
@@ -110,30 +110,6 @@ static int netlink_bufsize, netlink_no_enobufs;
 // #define TRACE(x...)
 // #endif
 
-static void sig_catcher(int sig)
-{
-#if !defined(NDEBUG)
-	if (ap.mode == 0)
-	{
-		char buf[80];
-		unsigned s = snprintf(buf, sizeof buf,
-			"%s [%d]: received signal %s\n",
-			err_prefix, (int)getpid(), strsignal(sig));
-		if (s >= sizeof buf)
-		{
-			s = sizeof buf;
-			buf[s - 1] = '\n';
-		}
-		write(2, buf, s);
-	}
-#endif
-	if (sig == SIGABRT)
-	{
-		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
-		abort();
-	}
-	caught_signal = sig;
-}
 
 struct ipaddr_cnt;
 typedef struct queue_descr
@@ -292,6 +268,18 @@ packet_cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
 static queue_descr default_queue[1], *queue_data = default_queue;
 static size_t queue_data_cnt = 1;
 
+static int get_buffer_size(int fd)
+{
+	socklen_t bufsize = 0, socklen = sizeof bufsize;
+	if (getsockopt(fd, SOL_SOCKET, SO_RCVBUF, &bufsize, &socklen))
+	{
+		report_error(&ap, LOG_CRIT, "getsockopt fail for fd %d (%s)\n",
+			fd, strerror(errno));
+		return -1;
+	}
+	return bufsize;
+}
+
 /*
 Possible verdicts and guessed behavior:
 
@@ -335,7 +323,7 @@ daemon_loop(struct nfq_handle *h, DB *db)
 	memset(&cb_cnt, 0, sizeof cb_cnt);
 
 	int const fd = nfq_fd(h);
-	int rv;
+	int rv = 0; // compiler happy
 	char buf[8192];
 	
 	if (fd < 0)
@@ -345,12 +333,39 @@ daemon_loop(struct nfq_handle *h, DB *db)
 		return 1;
 	}
 
+	if (verbose)
+	{
+		int bufsize = get_buffer_size(fd);
+		if (bufsize >= 0)
+			report_error(&ap, LOG_INFO, "netlink socket recv size %0x\n",
+				bufsize);
+	}
+
 	if (netlink_bufsize)
 	{
+		socklen_t bufsize = netlink_bufsize;
+		if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &bufsize, sizeof bufsize))
+			report_error(&ap, LOG_CRIT,
+				"setsockopt fail for fd %d size = %0x (%s)\n",
+					fd, netlink_bufsize, strerror(errno));
+		else if (verbose)
+		{
+			int newsize = get_buffer_size(fd);
+			if (newsize >= 0)
+				report_error(&ap, LOG_INFO,
+					"netlink socket recv size set to %0x, now %0x\n",
+					netlink_bufsize, newsize);
+		}
+
+		/* was:
 		unsigned int const read_size =
 			nfnl_rcvbufsiz(nfq_nfnlh(h), netlink_bufsize);
 		report_error(&ap, LOG_INFO, "netlink socket recv size size set to %u\n",
 			read_size);
+		*
+		* by replacing the above code, can compile without -lnfnetlink
+		* (which is set by pkg-config anyway)
+		*/
 	}
 
 	if (netlink_no_enobufs)
@@ -583,35 +598,6 @@ daemon_loop(struct nfq_handle *h, DB *db)
 	return rv;
 }
 
-static int setsigs(void)
-{
-	int rtc;
-	struct sigaction act;
-	memset(&act, 0, sizeof act);
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-	act.sa_handler = sig_catcher;
-
-	rtc =
-		sigaction(SIGPIPE, &act, NULL) ||
-		sigaction(SIGILL, &act, NULL)  ||
-		sigaction(SIGHUP, &act, NULL)  ||
-		sigaction(SIGINT, &act, NULL)  ||
-		sigaction(SIGQUIT, &act, NULL) ||
-		sigaction(SIGPWR,  &act, NULL) ||
-		sigaction(SIGABRT, &act, NULL) ||
-		sigaction(SIGTERM, &act, NULL);
-
-	act.sa_handler = SIG_IGN;
-	rtc |=
-		sigaction(SIGALRM, &act, NULL) ||
-		sigaction(SIGIO,   &act, NULL) ||
-		sigaction(SIGUSR1, &act, NULL) ||
-		sigaction(SIGUSR2, &act, NULL);
-//		sigaction(SIGEMT,  &act, NULL);
-	return rtc;
-}
-
 static char *db_name = IPQBDB_DATABASE_NAME;
 static int version_opt, help_opt, no_daemon_opt, no_cleanup_opt;
 static struct poptOption opttab[] =
