@@ -3,7 +3,7 @@
 * written by ale in milano on 28 sep 2008
 * read from a socket and extract pcre patterns to add to block.db
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -133,7 +133,6 @@ static app_private ap;
 static char const err_prefix[] = "ibd-parse";
 #include "logline_written_block.h"
 
-int caught_signal;
 
 #if !defined NDEBUG
 #define TRACE(x...) while (ap.mode == 0) { fprintf(stderr, ## x); break; }
@@ -173,60 +172,20 @@ static void child_reaper(int sig)
 	}
 }
 
-static void sig_catcher(int sig)
-{
-#if !defined(NDEBUG)
-	if (ap.mode == 0)
-	{
-		char buf[80];
-		unsigned s = snprintf(buf, sizeof buf,
-			"%s [%d]: received signal %s\n",
-			err_prefix, (int)getpid(), strsignal(sig));
-		if (s >= sizeof buf)
-		{
-			s = sizeof buf;
-			buf[s - 1] = '\n';
-		}
-		write(2, buf, s);
-	}
-#endif
-	if (sig == SIGABRT)
-		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
-	caught_signal = sig;
-}
+#define SETSIG_FUNC_CATCH_SIGUSR1
+#include "setsig_func.h"
 
-static int setsigs(void)
+static int my_setsigs(void)
 {
-	int rtc;
 	struct sigaction act;
 	memset(&act, 0, sizeof act);
 	sigemptyset(&act.sa_mask);
-		
+
 	act.sa_flags = SA_NOCLDSTOP;
 	act.sa_handler = child_reaper;
-	sigaction(SIGCHLD, &act, NULL);		
+	sigaction(SIGCHLD, &act, NULL);
 
-	act.sa_flags = 0;
-	act.sa_handler = sig_catcher;
-
-	rtc =
-		sigaction(SIGPIPE, &act, NULL) ||
-		sigaction(SIGILL, &act, NULL)  ||
-		sigaction(SIGPWR,  &act, NULL) ||
-		sigaction(SIGABRT, &act, NULL) ||
-		sigaction(SIGHUP, &act, NULL)  ||
-		sigaction(SIGUSR1, &act, NULL) ||
-		sigaction(SIGINT, &act, NULL)  ||
-		sigaction(SIGQUIT, &act, NULL) ||
-		sigaction(SIGTERM, &act, NULL);
-
-	act.sa_handler = SIG_IGN;
-	rtc |=
-		sigaction(SIGALRM, &act, NULL) ||
-		sigaction(SIGIO,   &act, NULL) ||
-		sigaction(SIGUSR2, &act, NULL);
-//		sigaction(SIGEMT,  &act, NULL);
-	return rtc;
+	return setsigs();
 }
 
 static int initial_count = IPQBDB_INITIAL_COUNT;
@@ -239,6 +198,7 @@ static char *pcre_name = IPQBDB_PCRE_FILE;
 static int version_opt, help_opt, no_cleanup_opt, chartable_opt, no_daemon_opt;
 static int exit_on_eof_opt, no_run_opt;
 static int verbose = -1;
+static int add_category_opt;
 static int exec_connkill_opt;
 static struct poptOption opttab[] =
 {
@@ -265,7 +225,9 @@ static struct poptOption opttab[] =
 	{"no-run", 'n', POPT_ARG_NONE, &no_run_opt, 0,
 	"Only check options and config files", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
-	"Be verbose", "level"},
+	"Be verbose if on terminal, log if daemon", "level"},
+	{"log-category", '\0', POPT_ARG_NONE, &add_category_opt, 0,
+	"Enable logging of the report category", NULL},
 	{"pcre-use-locale", 'l', POPT_ARG_NONE, &chartable_opt, 0,
 	"Build pcre character tables using current locale settings", NULL},
 	{"no-db-cleanup", '\0', POPT_ARG_NONE, &no_cleanup_opt, 0,
@@ -580,7 +542,7 @@ static int insert_pcre(initialization_stuff *ini,
 				pcre_fullinfo(r.code, r.extra, PCRE_INFO_STUDYSIZE, &studysize);
 			if (rc2 == 0)
 			{
-				char firststr[10];
+				char firststr[16];
 				if (firstbyte >= 0)
 					if (isgraph(firstbyte))
 						sprintf(firststr, "'%c'", firstbyte);
@@ -905,7 +867,7 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 		{
 			if (no_daemon_opt == 0)
 				openlog(err_prefix, LOG_PID, ap.mode = LOG_DAEMON);
-			if (setsigs())
+			if (my_setsigs())
 				what = "sigaction";
 			else if (chdir(ini.home_dir))
 				what = "chdir";
@@ -1139,7 +1101,8 @@ static int block_matched(runtime_stuff *run, re_list_t const * const r,
 		{
 			int probability = r->init_prob;
 			rtc = write_block(run->db[r->db_ndx], ip_addr, decay,
-				r->reason, &probability, 0,
+				r->reason, &probability,
+				add_category_opt? write_block_force_add_category: 0,
 				verbose? logline_written_block: NULL, run->db1);
 
 			if (rtc != 0)
@@ -1456,7 +1419,7 @@ int main(int argc, char const *argv[])
 			fputs("\n"
 			"The filename is that of the block database, subject to prefix/suffix\n"
 			"arrangements.  All DB filenames must end up in the same directory,\n"
-			"which is also used for\nthe socket.\n"
+			"which is also used for the socket.\n"
 			"\n"
 			"The decay defines the halving period, in seconds, and overrides the\n"
 			"initial-decay (see option -t above) for each expression/reason.\n"
@@ -1551,6 +1514,9 @@ int main(int argc, char const *argv[])
 							rtc = 0;
 							break;
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wimplicit-fallthrough="
+
 						case SIGINT:
 						case SIGQUIT:
 						case SIGTERM:
@@ -1562,6 +1528,8 @@ int main(int argc, char const *argv[])
 								strsignal(caught_signal), live_children, all_children);
 							rtc = exit_rtc;
 							break;
+#pragma GCC diagnostic pop
+
 					}
 					caught_signal = 0;
 				}
