@@ -3,7 +3,7 @@
 * written by ale in milano on 17 dec 2008
 * edit entries of white.db
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -55,58 +55,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 static app_private ap;
 static char const err_prefix[] = "ibd-white";
-int caught_signal;
-
-static void sig_catcher(int sig)
-{
-#if !defined(NDEBUG)
-	if (ap.mode == 0)
-	{
-		char buf[80];
-		unsigned s = snprintf(buf, sizeof buf,
-			"%s [%d]: received signal %s\n",
-			err_prefix, (int)getpid(), strsignal(sig));
-		if (s >= sizeof buf)
-		{
-			s = sizeof buf;
-			buf[s - 1] = '\n';
-		}
-		write(2, buf, s);
-	}
-#endif
-	if (sig == SIGABRT)
-		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
-	caught_signal = sig;
-}
-
-static int setsigs(void)
-{
-	int rtc;
-	struct sigaction act;
-	memset(&act, 0, sizeof act);
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-	act.sa_handler = sig_catcher;
-
-	rtc =
-		sigaction(SIGPIPE, &act, NULL) ||
-		sigaction(SIGILL, &act, NULL)  ||
-		sigaction(SIGHUP, &act, NULL)  ||
-		sigaction(SIGINT, &act, NULL)  ||
-		sigaction(SIGQUIT, &act, NULL) ||
-		sigaction(SIGPWR,  &act, NULL) ||
-		sigaction(SIGABRT, &act, NULL) ||
-		sigaction(SIGTERM, &act, NULL);
-
-	act.sa_handler = SIG_IGN;
-	rtc |=
-		sigaction(SIGALRM, &act, NULL) ||
-		sigaction(SIGIO,   &act, NULL) ||
-		sigaction(SIGUSR1, &act, NULL) ||
-		sigaction(SIGUSR2, &act, NULL);
-//		sigaction(SIGEMT,  &act, NULL);
-	return rtc;
-}
+#include "setsig_func.h"
 
 static char *db_white_name = IPQBDB_WHITE_DATABASE_NAME;
 static double default_decay = (IPQBDB_INITIAL_DECAY/2);
