@@ -1,7 +1,7 @@
 /*
-* percent_prob.h
+* logline_written_block.h
 
-Copyright (C) 2019 Alessandro Vesely
+Copyright (C) 2019-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -30,13 +30,17 @@ This is the line logging for ibd-parse -v and ibd-ban -l.
 */
 
 static void logline_written_block(uint32_t ip_addr, int reason_id,
+	int write_block_force,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db1)
 {
 	struct in_addr in;
 	memcpy(&in, &ip_addr, sizeof in);
 
-	char descr[IPQBDB_DESCR_MAX_LENGTH];
-	get_descr(db1, reason_id, 0, descr, sizeof descr);
+	char descr[IPQBDB_DESCR_MAX_LENGTH + 5];
+	get_descr(db1, reason_id,
+		(write_block_force & write_block_force_add_category)?
+			get_descr_add_category: 0,
+		descr, sizeof descr);
 
 	report_error(&ap, LOG_INFO,
 		"%s record for %s, decay: %g, prob: %.2f%%, %s\n",
