@@ -1,7 +1,7 @@
 /*
 * opendb2.c
 
-Copyright (C) 2008-2019 Alessandro Vesely
+Copyright (C) 2008-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -265,7 +265,8 @@ int get_descr(DB *db, int id, int opt, char *dest, size_t size)
 	} u;
 	char *const descr = &u.drec.descr[0];
 	char idbuf[32];
-	size_t dlen = 0, ilen = 0, alloc = 0;
+	char catbuf[32];
+	size_t dlen = 0, ilen = 0, clen = 0, alloc = 0;
 	int rtc;
 
 	if (db)
@@ -320,15 +321,18 @@ int get_descr(DB *db, int id, int opt, char *dest, size_t size)
 		opt |= get_descr_add_id;
 	}
 
+	if (rtc == 0 && (opt & get_descr_add_category))
+	{
+		clen = sprintf(catbuf, "%d", u.drec.report_category);
+		alloc += clen + 2; // comma and separating space
+	}
+
 	if (alloc >= size)
 		goto error_exit;
 
-	if (rtc == 0 && (opt & get_descr_quoted) && dlen > 0)
-	{
-		alloc += 2; // quotes only around original
-	}
-	else if (rtc || dlen == 0)
+	if (rtc || dlen == 0)
 	/*
+	* Record not found or data length == 0:
 	* meta-description, parethesized if descr not quoted
 	*/
 	{
@@ -351,6 +355,10 @@ int get_descr(DB *db, int id, int opt, char *dest, size_t size)
 		dlen = strlen(descr);
 		opt &= ~get_descr_quoted;
 	}
+	else if (opt & get_descr_quoted)
+	{
+		alloc += 2; // quotes only around original
+	}
 
 	/*
 	* truncate description if too long.
@@ -392,6 +400,13 @@ int get_descr(DB *db, int id, int opt, char *dest, size_t size)
 		memcpy(dest, idbuf, ilen);
 		dest += ilen;
 	}
+	if (opt & get_descr_add_category)
+	{
+		*dest++ = ',';
+		*dest++ = ' ';
+		memcpy(dest, catbuf, clen);
+		dest += clen;
+	}
 	*dest = 0;
 	return 0;
 
@@ -636,7 +651,7 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 			set_descr_cnt(db_d, reason_id, !old, NULL, &now);
 			
 		if (verbose_fn)
-			verbose_fn(ip_addr, reason_id,
+			verbose_fn(ip_addr, reason_id, force,
 				&ip_data, old? &old_data: NULL, db_d);
 	}
 	
