@@ -1,7 +1,7 @@
 /*
 * percent_prob.h
 
-Copyright (C) 2011 Alessandro Vesely
+Copyright (C) 2011-2021 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -23,11 +23,14 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <stdlib.h>
 
-static inline double percent_prob(int p)
+static inline double percent_prob(unsigned int p)
 /*
 * Return the probability as a percentage, suitable for display
 */
 {
+	if (p > RAND_MAX)
+		p = RAND_MAX;
+
 	return 100.0 * (double)p / ((double)RAND_MAX);
 }
 
