@@ -53,7 +53,9 @@ call to recv terminate with EBADF. nfq_fd will return -123 after that.
 
 */
 
+#if ! defined _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -106,7 +108,7 @@ struct nfq_data
 			unsigned source:32;
 			unsigned dest:32;
 		} v;
-		char payload[20]; // extra header and data would follow
+		unsigned char payload[20]; // extra header and data would follow
 	} packet;
 	int packet_id;
 	struct nfqnl_msg_packet_hdr hdr;
@@ -958,9 +960,12 @@ int nfq_set_queue_maxlen(struct nfq_q_handle *qh, u_int32_t queue_maxlen)
 	return 0;
 }
 
-int nfq_set_verdict_mark(struct nfq_q_handle *qh, u_int32_t id,
-		u_int32_t verdict, u_int32_t mark,
-		u_int32_t datalen, unsigned char *buf)
+int nfq_set_verdict2(struct nfq_q_handle *qh,
+                            u_int32_t id,
+                            u_int32_t verdict, 
+                            u_int32_t mark,
+                            u_int32_t datalen,
+                            const unsigned char *buf)
 {
 	if (get_queue_num(qh) < 0)
 		return -1;
@@ -972,9 +977,11 @@ int nfq_set_verdict_mark(struct nfq_q_handle *qh, u_int32_t id,
 	(void)buf;
 }
 
-int nfq_set_verdict(struct nfq_q_handle *qh, u_int32_t id,
-		u_int32_t verdict, u_int32_t data_len, 
-		unsigned char *buf)
+int nfq_set_verdict(struct nfq_q_handle *qh,
+                             u_int32_t id,
+                             u_int32_t verdict,
+                             u_int32_t data_len,
+                             const unsigned char *buf)
 {
 	if (get_queue_num(qh) < 0)
 		return -1;
@@ -1044,7 +1051,7 @@ struct nfqnl_msg_packet_hw *nfq_get_packet_hw(struct nfq_data *pkt)
 	return 0;
 }
 
-int nfq_get_payload(struct nfq_data *pkt, char **data)
+int nfq_get_payload(struct nfq_data *pkt, unsigned char **data)
 {
 	*data = pkt->packet.payload;
 	return sizeof pkt->packet.payload;
